<?php

namespace Repository;

use Doctrine\ORM\EntityManager;
use Util\InstanceProvider;
use Teknisa\Model\Traits\TQuery;
use Zeedhi\DevOps\Util\Utilities;

class TaskTracking {
    use TQuery;
    /**
     * @var EntityManager
     */
    private $entityManager;

    /**
     * TaskTracking constructor.
     * @param EntityManager $entityManager
     */
    public function __construct(EntityManager $entityManager)
    {
        $this->entityManager = $entityManager;
    }

    public function getAllTasksByProjectOnDemand($organizationId,$filters = '',$filterLabel = '', $filterIssue = '',
                                                 $project = false, $statusResource = false, $filterSearchQuery, $page, $pageSize) {
        $filterTypeProject = Utilities::queryFilterTypeProjectMeeting('PT',true);
        $filterLabelIssue = '';
        if($filterLabel) {
            $filterLabelIssue .= " AND TAS.ID IN (SELECT TRE.TASK_RELATIONSHIP
                                                    FROM TASK_LABEL TLA
                                               LEFT JOIN TASK_RELATIONSHIP TRE ON (TRE.TASK = TLA.TASK AND NVL(TRE.TYPE, 'I') = 'I')
                                                   WHERE TLA.LABEL IN($filterLabel))";
        }
        if($filterIssue) {
            $filterLabelIssue .= " AND TAS.ID IN (SELECT TASK_RELATIONSHIP
                                                    FROM TASK_RELATIONSHIP
                                                   WHERE TASK_RELATIONSHIP = TAS.ID
                                                     AND NVL(TYPE, 'I') = 'I'
                                                     AND TASK IN ($filterIssue))";
        }
        if($statusResource) {
            $filterLabelIssue .= " AND RES.STATUS = '$statusResource'";
        }
        $startPos = ($page - 1) * $pageSize;
        $finalPos = $page * $pageSize;
        $query = <<<SQL
        SELECT COUNT(1) OVER () TOTAL_LINES_ZHNEXT, C.* FROM (
           SELECT TASK_NAME,PROJECT_NAME||TASK_NAME AS ORDER_BY, NAME_RESOURCE,MODEL_ID,RESOURCE_T,
                (SELECT RTRIM(XMLAGG(XMLELEMENT(E,RP.EMAIL,',').EXTRACT('//text()') ORDER BY RP.ID).GETCLOBVAL(),',')
                     FROM RESOURCE_T RP
                    INNER JOIN ALLOCATION AL ON (RP.ID = AL.RESOURCE_T)
                    WHERE TASK_ID = AL.TASK) AS EMAIL_RESOURCE,
                  ESTIMATED_BEGIN_DATE, ESTIMATED_END_DATE,DURATION,SUM(TOTAL_APPROPRIATION) AS TOTAL_APPROPRIATION,
                  LABEL AS TASK_STATUS_LABEL,TIME_PLANNED,PERCENT_COMPLETED||'%' AS PERCENT_COMPLETED,TASK_ID,PROJECT_ID, PROJECT_NAME
                  FROM(SELECT  TASK_NAME,NAME_RESOURCE,ESTIMATED_BEGIN_DATE, ESTIMATED_END_DATE,DURATION,MODEL_ID,RESOURCE_T,
                              TOTAL_APPROPRIATION,LABEL,PERCENT_COMPLETED,TIME_PLANNED,TASK_ID,PROJECT_ID,EMAIL_RESOURCE, PROJECT_NAME, NULL AS TASK_STATUS_LABEL
                       FROM (SELECT NULL AS TASK_STATUS_LABEL, TAS.ID AS TASK_ID,TAS.NAME AS TASK_NAME,RES.NAME AS NAME_RESOURCE,TLA.LABEL AS TASK_LABEL,
                                  TO_CHAR(TAS.ESTIMATED_BEGIN_DATE, 'YYYYMMDD') AS ORDER_FRONT, RES.EMAIL AS EMAIL_RESOURCE,
                                  TO_CHAR(TAS.ESTIMATED_BEGIN_DATE, 'DD/MM/YYYY') AS ESTIMATED_BEGIN_DATE,
                                  TO_CHAR(TAS.ESTIMATED_END_DATE, 'DD/MM/YYYY') AS ESTIMATED_END_DATE,
                                  TAS.DURATION,TAS.TOTAL_APPROPRIATION AS TOTAL_APPROPRIATION,TST.LABEL,TAS.PERCENT_COMPLETED,
                                  TAS.PROJECT_ID AS ID,TAS.PROJECT_ID, TAS.PROJECT_ID_FILTER, PRO.NAME AS PROJECT_NAME,TAS.STATUS,
                                  TST.ID AS TASK_STATUS, RES.ID AS RESOURCE_T,PER.STRUCTURE,
                                  DECODE(TK.ID, NULL, TAS.TIME_PLANNED, 0) AS TIME_PLANNED,
                                  (   SELECT PRM.ID
                                        FROM PROJECT PRO1
                                        JOIN PROJECT_RELATIONSHIP PRR ON (PRO1.ID = PRR.PROJECT AND PRR.TYPE = 'G')
                                        JOIN PROJECT PRM ON(PRM.ID = PRR.PROJECT_RELATIONSHIP)
                                        WHERE PRO1.ID = PRO.ID AND ROWNUM = 1) AS MODEL_ID,
                                  ALO.RESOURCE_T AS TASK_RESOURCE,PRE.RESOURCE_C AS MANAGER_RESOURCE,
                               TAS.ESTIMATED_BEGIN_DATE AS PERIOD,TAS.ESTIMATED_END_DATE AS PERIOD_END, TASK_CLASSIFICATION, TAS.CREATED_AT
                             FROM ( SELECT  TAS.ID AS TASK_ID,TAS.PROJECT AS PROJECT_ID_FILTER,TAS.ID,TAS.ORGANIZATION,TAS.NAME,TAS.ESTIMATED_BEGIN_DATE,TAS.ESTIMATED_END_DATE,
                                             TAS.PERCENT_COMPLETED,TAS.DURATION,TAS.TIME_PLANNED,TAS.PROJECT AS PROJECT_ID,TAS.STATUS, TAS.TASK_STATUS, TAS.CLASSIFICATION AS TASK_CLASSIFICATION,
                                             SUM(APR.TOTAL) AS TOTAL_APPROPRIATION, TAS.CREATED_AT
                                             FROM TASK TAS
                                                      LEFT JOIN APPROPRIATION APR ON(APR.TASK = TAS.ID)
                                                      WHERE TAS.ORGANIZATION = :organizationId
                                                      AND TAS.STATUS <> 'D'
                                                      AND NVL(TAS.IS_PARENT,'N') <> 'Y'
                                                      AND NVL(TAS.IS_ISSUE,'N') <> 'Y'
                                                      GROUP BY TAS.ID, TAS.ORGANIZATION, TAS.TIME_PLANNED, TAS.CALENDAR, TAS.PROJECT, TAS.NAME,
                                             TAS.ID,TAS.ORGANIZATION,TAS.NAME,TAS.ESTIMATED_BEGIN_DATE,TAS.ESTIMATED_END_DATE,
                                             TAS.PERCENT_COMPLETED,TAS.DURATION,TAS.PROJECT,TAS.STATUS, TAS.TASK_STATUS, TAS.CLASSIFICATION, TAS.CREATED_AT ) TAS
                        LEFT JOIN ALLOCATION ALO ON (ALO.TASK = TAS.ID)
                        LEFT JOIN RESOURCE_T RES ON(RES.ID = ALO.RESOURCE_T)
                        LEFT JOIN TASK_STATUS TST ON(TST.ID = ALO.TASK_STATUS)
                        LEFT JOIN PROJECT PRO ON(PRO.ID = TAS.PROJECT_ID)
                        LEFT JOIN RESOURCE_PROJECT PRE ON(PRO.ID = PRE.PROJECT)
                        LEFT JOIN TASK_LABEL TLA ON(TLA.TASK = TAS.ID)
                        LEFT JOIN PROJECT_TYPE PT ON (PT.ID = PRO.PROJECT_TYPE)
                        LEFT JOIN PERMISSION PER ON (PER.RESOURCE_T = RES.ID AND  NVL(PER.TYPE,'R') = 'R')
                        LEFT JOIN TASK_RELATIONSHIP TK ON (TK.TASK_RELATIONSHIP = TAS.ID AND TK.TYPE = 'A')
                            WHERE TAS.ORGANIZATION = :organizationId
                              $filterTypeProject
                              $filterLabelIssue)
                        WHERE 1=1
                          $filters
                          GROUP BY  TASK_NAME,ESTIMATED_BEGIN_DATE,NAME_RESOURCE, ESTIMATED_END_DATE,DURATION,LABEL,RESOURCE_T,
                          PERCENT_COMPLETED,TASK_ID,PROJECT_ID,TOTAL_APPROPRIATION,EMAIL_RESOURCE,TIME_PLANNED, PROJECT_NAME,MODEL_ID, CREATED_AT)
          GROUP BY TASK_NAME, ESTIMATED_BEGIN_DATE, ESTIMATED_END_DATE, DURATION, TOTAL_APPROPRIATION,TIME_PLANNED,
                    LABEL, PERCENT_COMPLETED, TASK_ID, PROJECT_ID, PROJECT_NAME,NAME_RESOURCE,MODEL_ID,RESOURCE_T
          ORDER BY ORDER_BY) C
          $filterSearchQuery
SQL;
        $params = array(
			'organizationId' => $organizationId
        );
        $query = Utilities::getPaginatedQuery($query,$finalPos,$startPos);
        $result =  $this->entityManager->getConnection()->fetchAll($query, $params);
        $total = empty($result) ? 0 : $result[0]['TOTAL_LINES_ZHNEXT'];
        $returnNext = [ 'data' => $result, 'pagination' => [ 'limit' => $pageSize, 'page' => $page, 'total' => $total]];
        return $returnNext;
    }
    
    public function getInfoTaskInTaskTracking($idTask,$organization,$resource = null) {

        $resourceFilter = '';
        if($resource){
            $resourceFilter = " AND RP.ID = $resource";
        }
		$query = <<<SQL
			SELECT 	TA.ID AS TASK_ID,TA.ID, TA.NAME,TA.NAME AS TASK_NAME, TO_CHAR(PRO.END_DATE,'YYYY-MM-DD') AS END_PROJECT, TO_CHAR(PRO.BEGIN_DATE,'YYYY-MM-DD') AS BEGIN_PROJECT,
					TO_CHAR(TA.ESTIMATED_BEGIN_DATE,'DD/MM/YYYY') AS "START", TAPR.NAME AS PRECURSOR_NAME, TP.TASK_RELATIONSHIP AS PRECURSOR,
        			TR.TASK AS TASK_GENERATOR, TAGE.NAME AS TASK_GENERATOR_NAME,
        			TR.TYPE AS TYPE_RELATIONSHIP,PRO.ID AS PROJECT_ID,
        			TO_CHAR(TA.ESTIMATED_END_DATE,'DD/MM/YYYY') AS "END", TAPA.NAME AS PARENT_NAME, TA.PERCENT_COMPLETED, TA.PARENT AS PARENT,
        			TA.PERCENT_COMPLETED as PERCENT,(SELECT C.LABEL FROM CLASSIFICATION C WHERE C.ID = TA.CLASSIFICATION) AS TASK_CLASSIFICATION_LABEL,
        			(CASE WHEN TA.STATUS = 'A' THEN 'Ativa' ELSE  'Inativa' END ) AS STATUS,
        			TO_CHAR(TA.DATE_EXCEPTION,'DD/MM/YYYY') AS DATE_EXCEPTION, TA.IS_PARENT,PRO.ID AS PROJECT,PRO.NAME AS PROJECT_NAME,
        			TA.PREVIOUS_TIME/3600 as PREVIOUS_TIME,TA.STATUS AS STATUS_CHECK_BOX,TA.RESPONSE AS TASK_RESPONSE,
        			TA.ERROR_CLASSIFICATION_ID,	EC.DESCRIPTION AS ERROR_CLASSIFICATION,
                    (
                		SELECT rtrim(xmlagg(xmlelement(e,RP.ID,',').extract('//text()') ORDER BY RP.ID).GetClobVal(),',')
                		FROM 		RESOURCE_T RP
                		INNER JOIN 	ALLOCATION AL ON (RP.ID = AL.RESOURCE_T)
                		WHERE TA.ID = AL.TASK
                		$resourceFilter
                	) AS "RESOURCE",
                    (
                		SELECT rtrim(xmlagg(xmlelement(e,RP.NAME,',').extract('//text()') ORDER BY RP.NAME).GetClobVal(),',')
                		FROM 		RESOURCE_T RP
                		INNER JOIN 	ALLOCATION AL ON (RP.ID = AL.RESOURCE_T)
                		WHERE TA.ID = AL.TASK
                		$resourceFilter
                	) AS "NAME_RESOURCE",
                    (
                		SELECT rtrim(xmlagg(xmlelement(e,TS.LABEL,',').extract('//text()') ORDER BY RP.NAME).GetClobVal(),',')
                		FROM 		RESOURCE_T RP
                		INNER JOIN 	ALLOCATION AL   ON (RP.ID = AL.RESOURCE_T)
                		LEFT JOIN 	TASK_STATUS TS 	ON (TS.ID = AL.TASK_STATUS)
                		WHERE TA.ID = AL.TASK
                		$resourceFilter
                	) AS TASK_STATUS_ALLOCATION_NAME,
                    (
                           SELECT   rtrim(xmlagg(xmlelement(e,tla.label,',').extract('//text()') ORDER BY tla.id).getclobval(),',')
                           FROM     task_label tla
                           WHERE    ta.id = tla.task) AS TASK_LABEL,
                           TP.TYPE_PRECURSOR AS TYPE, TA.LATENCY, TA.DESCRIPTION, TS.ID AS TASK_STATUS, TS.LABEL AS TASK_STATUS_NAME,
					TA.DURATION AS TOTAL_TIME, TT.ID AS TASK_TYPE, TA.DATE_EXCEPTION_TYPE AS DATE_EXCEPTION_ABBREVIATION, TA.DATE_EXCEPTION_TYPE,
					PT.USETIMEPLANNING,
                    DECODE(TK.ID, NULL, TA.TIME_PLANNED, 0) AS TIME_PLANNED,
					RCB.NAME AS CREATED_BY,TO_CHAR(TA.CREATED_AT,'DD/MM/YYYY HH24:MI:SS') AS CREATED_AT
            FROM 		TASK TA
            LEFT JOIN 	PROJECT PRO 			ON (TA.PROJECT = PRO.ID)
            LEFT JOIN 	PROJECT_TYPE PT 		ON (PT.ID = PRO.PROJECT_TYPE)
	        LEFT JOIN 	TASK_RELATIONSHIP TP 	ON (TA.ID = TP.TASK AND TP.TYPE = 'P')
	        LEFT JOIN 	TASK_RELATIONSHIP TR 	ON (TA.ID = TR.TASK_RELATIONSHIP AND TR.TYPE IN ('C', 'E', 'R', 'M','T'))
            LEFT JOIN 	TASK_STATUS TS 			ON (TS.ID = TA.TASK_STATUS)
            LEFT JOIN 	TASK_TYPE TT 			ON (TT.ID = TS.TASK_TYPE)
            LEFT JOIN 	TASK TAPR 				ON (TAPR.ID = TP.TASK_RELATIONSHIP)
            LEFT JOIN 	TASK TAGE 				ON (TAGE.ID = TR.TASK)
            LEFT JOIN 	TASK TAPA 				ON (TAPA.ID = TA.PARENT)
            LEFT JOIN   RESOURCE_T RCB          ON (RCB.ID = TA.CREATED_BY)
            LEFT JOIN   ERROR_CLASSIFICATION EC ON EC.ID = TA.ERROR_CLASSIFICATION_ID
            LEFT JOIN   TASK_RELATIONSHIP TK    ON (TK.TASK_RELATIONSHIP = TA.ID AND TK.TYPE = 'A')
            WHERE TA.ORGANIZATION = :organization
            AND TA.ID = :idTask
SQL;
		$params = array(
			'organization' => $organization,
			'idTask' => $idTask
		);

		$tasks = $this->entityManager->getConnection()->fetchAll($query, $params);

		foreach($tasks as &$task){
			$task['RESOURCE'] = explode(',',$task['RESOURCE']);
			$task['TASK_LABEL'] = explode(',',$task['TASK_LABEL']);
			$task['TOTAL_TIME'] = Utilities::secondsToTime($task['TOTAL_TIME']);
            
			if(InstanceProvider::getRepository()->getTask()->findOneBy(array('parent' => $idTask))){
				$task['IS_UPPER'] = true;
			} else {
				$task['IS_UPPER'] = false;
			}
		}

		return $tasks;
	}

    public function findAllStatusTaskTracking($organizationId, $allowEndTasks = 'Y', $filterString = '') {
		$query = <<<SQL
        SELECT 	ID, LABEL, CONTROL,
            (CASE CONTROL WHEN 'N' THEN 1
            WHEN 'D' THEN 2
            WHEN 'P' THEN 3
            WHEN 'C' THEN 4
            END) ORDER_BY
		FROM 	TASK_STATUS
		WHERE 	ORGANIZATION = :ORGANIZATION_ID
		AND 	USED_IN_TASK = 'Y'
		AND 	IS_PROJECT = 'N'
		$filterString
SQL;
		$params = array(
			'ORGANIZATION_ID' => $organizationId,
		);

		return $this->createAndExecuteQueryAndBuildResponseData(InstanceProvider::getEntityManager(), $query, $params);
	}

    public function getAllIssuesFromFilterTaskTracking($organizationId, $conditions = ''){
		$query = <<<SQL
			SELECT ID,NAME FROM(SELECT TAS.ID,TAS.NAME,PRO.NAME AS PROJECT
                     FROM TASK TAS
                     LEFT JOIN PROJECT PRO  ON(PRO.ID = TAS.PROJECT)
                    WHERE TAS.ORGANIZATION = :organizationId
                      AND TAS.STATUS <> 'D'
                      AND TAS.IS_ISSUE = 'Y')
                      WHERE 1=1
                      $conditions
                      GROUP BY ID,NAME
                      ORDER BY ID
SQL;

		$params = array(
			'organizationId' => $organizationId
        );

        return $this->createAndExecuteQueryAndBuildResponseData(InstanceProvider::getEntityManager(), $query, $params);
	}

  public function findLabelsByFilter($organizationId, $userId){

		$query = <<<SQL
		SELECT * FROM(
			SELECT LAB.*,PRO.ID AS PROJECT_ID,PRO.NAME AS PROJECT_NAME,PRT.ID AS PROJECT_TYPE_ID,PRT.LABEL AS PROJECT_TYPE_LABEL
			FROM LABEL LAB
            LEFT JOIN PROJECT PRO ON (PRO.ID = LAB.PROJECT)
			LEFT JOIN PROJECT_TYPE PRT ON(PRT.ID = PRO.PROJECT_TYPE)
			WHERE LAB.ORGANIZATION = :organizationId
			AND (LAB.RESOURCE_T = :userId OR LAB.RESOURCE_T IS NULL))
			WHERE 1 = 1
SQL;
		$params = array(
			'organizationId' => $organizationId,
			'userId' => $userId
		);

		  return $this->createAndExecuteQueryAndBuildResponseData(InstanceProvider::getEntityManager(), $query, $params);
	}

  public function findLabelsByProjectTaskTracking($organizationId, $userId){
    if(empty($projectId)){
        $projects = '0';
    }else{
        if(is_array($projectId)){
            $projects = implode(',',$projectId);
        }else{
            $projects =  $projectId;
        }
    }
    $query = <<<SQL
SELECT * FROM (
  SELECT LAB.*,PRO.ID AS PROJECT_ID,PRO.NAME AS PROJECT_NAME,PRT.ID AS PROJECT_TYPE_ID,PRT.LABEL AS PROJECT_TYPE_LABEL
  FROM LABEL LAB
  LEFT JOIN PROJECT PRO ON(PRO.ID = LAB.PROJECT)
  LEFT JOIN PROJECT_TYPE PRT ON(PRT.ID = PRO.PROJECT_TYPE)
  WHERE LAB.ORGANIZATION = :organizationId
  AND (LAB.RESOURCE_T = :userId OR LAB.RESOURCE_T IS NULL)
  AND (LAB.PROJECT IN ( $projects ) OR LAB.PROJECT IS NULL))
  WHERE 1=1
SQL;
    $params = array(
        'organizationId' => $organizationId,
        'userId' => $userId
    );
    return $this->createAndExecuteQueryAndBuildResponseData(InstanceProvider::getEntityManager(), $query, $params);
}

}