<?php


namespace Service;

use Util\InstanceProvider;
use Zeedhi\Framework\Session\Session;
use \Util\Utilities;
use Exception;
use \Doctrine\ORM\EntityManager;
use Zeedhi\DevOps\Util\Utilities as DevOpsUtilities;

class StructureConfiguration
{
    /**
     * @var Utilities
     */
    private $environment;
     /**
     * @var EntityManager
     */
    public $entityManager;

    public function __construct()
    {
        $this->environment    = InstanceProvider::getEnvironment();
        $this->entityManager  = InstanceProvider::getEntityManager();

    }

    public function findStructureConfiguration(){
        $nrOrg          = Utilities::getEnvironment()->getCurrentOrganizationId();
        $structureConfigurationRepository = InstanceProvider::getRepository()->getStructureConfiguration();

        return $structureConfigurationRepository->findStructureConfiguration($nrOrg);
    }

    public function saveStructureConfiguration($rows){
        $connection = $this->entityManager->getConnection();
        try {
            $connection->beginTransaction();
            $cdOper = $this->environment->getCurrentCdoperador();
            $nrOrg          = Utilities::getEnvironment()->getCurrentOrganizationId();
            foreach($rows as $row){
                $id = empty($row['ID']) ? null : $row['ID'];
                if($id){
                    $structureType = InstanceProvider::getRepository()->getStructureType()->find($id);
                    $StructureType  = InstanceProvider::getFactory()->StructureType()->updateStructureType($structureType, $row['LABEL'], $row['CONTROL'], $cdOper, $nrOrg);
                }else{
                    $StructureType  = InstanceProvider::getFactory()->StructureType()->createStructureType($row['LABEL'], $row['CONTROL'], $cdOper, $nrOrg);
                }   
                $this->entityManager->persist($StructureType);
                $this->entityManager->flush();
            }
            
            $connection->commit();

            
            return $StructureType->getId();
        } catch (Exception $e) {
            try {
                $connection->rollBack();
            } catch (Exception $e1) {
            }
            throw $e;
        }
    }

    public function removeStructureType($rows){
        $connection = $this->entityManager->getConnection();
        try {
            $connection->beginTransaction();
            foreach($rows as $row){
                $structureType = InstanceProvider::getRepository()->getStructureType()->find($row['ID']);
                $this->entityManager->remove($structureType);
                $this->entityManager->flush();
            }
            $connection->commit();
        } catch (Exception $e) {
            throw $e;
        }
    }

    public function checkStructureTypeInUser($rows){
        $names = '';
        $return = false;
        foreach($rows as $row){
            $structures = $this->entityManager->getRepository('\Zeedhi\DevOps\Model\Projectmanagement\Entities\Structure')->
                    findBy(array('structureType' =>$row['ID']));
            if(!empty($structures)){
                foreach ($structures as $structure){
                    $names .=  $structure->getName().', ';
                }
            }
        }
        if($names){
            $return = 'Tipo sendo usado na(s) estrutura(s) '.substr($names, 0, -2);
        }
        return $return;
    }
}